/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { type } from 'os';
import { AllLogicStatement, AnyStatement, InputNodeType } from './logic/general';
import { MLTypes } from '../../data-access/store/mlSlice';

/** JSON query format used to send a query to the backend. */
export interface BackendQueryResultFormat {
  databaseName: string;
  return: {
    entities: number[];
    relations: number[];
    groupBys: number[];
  };
  entities: Entity[];
  relations: Relation[];
  // groupBys: GroupBy[];
  // machineLearning: MachineLearning[];
  limit: number;
  // modifiers: ModifierStruct[];
  // prefix: string;
}

/** JSON query format used to send a query to the backend. */
export interface BackendQueryFormat {
  databaseName: string;
  limit: number;
  return: string[];
  query: QueryStruct[];
  logic?: AnyStatement;
  // entities: Entity[];
  // relations: Relation[];
  // groupBys: GroupBy[];
  machineLearning: MachineLearning[];
  // modifiers: ModifierStruct[];
  // prefix: string;
}

/** Interface for an entity in the JSON for the query. */
export interface QueryStruct {
  ID: string;
  node: NodeStruct;
}

export interface NodeStruct {
  label?: string;
  ID?: string;
  logic?: AllLogicStatement;
  filter?: FilterStruct[];
  relation?: RelationStruct;
  subQuery?: QueryStruct;
  export?: ExportNodeStruct[];
}

export interface ExportNodeStruct {
  ID: string;
  attribute: string;
}

export interface LogicStruct {
  ID: string;
  attribute: string;
  operation: LogicOperationType;
}

export interface FilterStruct {
  attribute: string;
  operation: FilterOperationType;
  value: string;
}

export interface RelationStruct {
  ID?: string;
  label?: string;
  depth: QuerySearchDepthStruct;
  direction: 'TO' | 'FROM' | 'BOTH';
  node?: NodeStruct;
}

export interface QuerySearchDepthStruct {
  min: number;
  max: number;
}

export type FilterOperationType = 'EQ' | 'NEQ' | 'GT' | 'LT' | 'GTE' | 'LTE' | 'LIKE' | 'NOT LIKE' | 'IN' | 'NOT IN';
export type LogicOperationType = 'AND' | 'OR';

/** Interface for an entity in the JSON for the query. */
export interface Entity {
  name: string;
  ID: number;
  constraints: Constraint[];
}

/** Interface for an relation in the JSON for the query. */
export interface Relation {
  name: string;
  ID: number;
  fromType: string;
  fromID: number;
  toType: string;
  toID: number;
  depth: { min: number; max: number };
  constraints: Constraint[];
}

// /** Interface for an relation in the JSON for the query. */
// export interface Relation {
//   name: string;
//   ID: number;
//   fromType: string;
//   fromID: number;
//   toType: string;
//   toID: number;
//   depth: { min: number; max: number };
//   constraints: Constraint[];
// }

// /** JSON query format used to send a query to the backend. */
// export interface TranslatedJSONQuery {
//   return: {
//     entities: number[];
//     relations: number[];
//     groupBys: number[];
//   };
//   entities: Entity[];
//   relations: Relation[];
//   groupBys: GroupBy[];
//   machineLearning: MachineLearning[];
//   limit: number;
// }

// ////////////////////

/**
 * Constraint datatypes created from the attributes of a relation or entity.
 *
 * string MatchTypes: exact/contains/startswith/endswith.
 * int    MatchTypes: GT/LT/EQ.
 * bool   MatchTypes: EQ/NEQ.
 */
export interface Constraint {
  attribute: string;
  dataType: string;

  matchType: string;
  value: string;
}

// /** Interface for a function in the JSON for the query. */
// export interface GroupBy {
//   ID: number;

//   groupType: string;
//   groupID: number[];
//   groupAttribute: string;

//   byType: string;
//   byID: number[];
//   byAttribute: string;

//   appliedModifier: string;
//   relationID: number;
//   constraints: Constraint[];
// }

// /** Interface for Machine Learning algorithm */
export interface MachineLearning {
  ID?: number;
  type: MLTypes;
  parameters?: string[];
}

// /** Interface for what the JSON needs for link predicition */
// export interface LinkPrediction {
//   queuename: string;
//   parameters: {
//     key: string;
//     value: string;
//   }[];
// }

// export interface ModifierStruct {
//   type: string;
//   selectedType: string;
//   selectedTypeID: number;
//   attributeIndex: number;
// }
