import React from 'react';
import ColorPicker from '../../shared/ColorPicker';

type Props = {
  layer: any;
  updatedLayer: any;
  graphInfo: any;
  deleteLayer: any;
};

export default function NodeOptions({ layer, updatedLayer, graphInfo, deleteLayer }: Props) {
  return (
    <div>
      <div className="divider m-0"></div>

      <div>
        <h5>Opacity</h5>
        <div>
          <input
            className="range"
            type="range"
            min={0}
            max={1}
            step={0.1}
            value={layer.config.opacity}
            onChange={(e: any) =>
              updatedLayer(layer.id, {
                config: { ...layer.config, opacity: e.target.value },
              })
            }
            aria-labelledby="input-slider"
          />
        </div>
      </div>

      <div className="divider m-0"></div>

      <div>
        <h5>Node color</h5>
        <div>
          <div className="form-control">
            <label className="label cursor-pointer">
              <span className="label-text">Fixed</span>
              <input
                className="checkbox"
                type="checkbox"
                checked={layer.config.fillColor.fixed}
                onChange={(e) =>
                  updatedLayer(layer.id, {
                    config: {
                      ...layer.config,
                      fillColor: {
                        ...layer.config.fillColor,
                        fixed: e.target.checked,
                      },
                    },
                  })
                }
              />
            </label>
          </div>

          {layer.config.fillColor.fixed && (
            <div>
              <h5>Color</h5>
              <ColorPicker
                value={layer.config.fillColor.value}
                updateValue={(newValue: any) =>
                  updatedLayer(layer.id, {
                    config: {
                      ...layer.config,
                      fillColor: {
                        ...layer.config.fillColor,
                        value: newValue,
                      },
                    },
                  })
                }
              />
            </div>
          )}

          {!layer.config.fillColor.fixed && (
            <div>
              <select
                className="select select-bordered w-full max-w-xs"
                value={layer.config.fillColor.condition}
                onChange={(e) =>
                  updatedLayer(layer.id, {
                    config: {
                      ...layer.config,
                      fillColor: {
                        ...layer.config.fillColor,
                        condition: e.target.value,
                      },
                    },
                  })
                }
              >
                <option disabled selected>
                  Attribute
                </option>
                {Object.keys(graphInfo.nodeAttributes).map((type) => (
                  <option key={type} value={type}>
                    {type}
                  </option>
                ))}
              </select>

              <div>
                <h5>Colors</h5>
                <ColorPicker
                  value={layer.config.fillColor.scaleRange[0]}
                  updateValue={(newValue: any) =>
                    updatedLayer(layer.id, {
                      config: {
                        ...layer.config,
                        fillColor: {
                          ...layer.config.fillColor,
                          scaleRange: [newValue, layer.config.fillColor.scaleRange[1]],
                        },
                      },
                    })
                  }
                />
                <ColorPicker
                  value={layer.config.fillColor.scaleRange[1]}
                  updateValue={(newValue: any) =>
                    updatedLayer(layer.id, {
                      config: {
                        ...layer.config,
                        fillColor: {
                          ...layer.config.fillColor,
                          scaleRange: [layer.config.fillColor.scaleRange[0], newValue],
                        },
                      },
                    })
                  }
                />
              </div>
            </div>
          )}
        </div>
      </div>

      <div className="divider m-0"></div>

      <div>
        <button
          className="btn btn-primary w-full"
          onClick={() =>
            updatedLayer(layer.id, {
              visible: !layer.visible,
            })
          }
        >
          {layer.visible ? 'Hide layer' : 'Show layer'}
        </button>

        <button className="btn btn-outline btn-primary w-full mt-1" onClick={() => deleteLayer(layer.id)}>
          Delete Layer
        </button>
      </div>
    </div>
  );
}
